/*
 * Copyright (c) 2001-2004 Marcin Dalecki and others
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * PAUL VOJTA OR ANY OTHER AUTHOR OF THIS SOFTWARE BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

/*
 * Implementation of the File selection dialogue for the Motif GUI.
 */

#include "xdvi-config.h"
#include "xdvi.h"
#include "sfSelFile.h"

#include "dvi.h"
#include "string-utils.h"
#include "util.h"
#include "events.h"
#include "message-window.h"
#include "dvi-init.h" /* for dviErrFlagT */

#if defined(SELFILE) && defined(MOTIF) /* entire file */

#include <Xm/FileSB.h>

static Widget dialog = NULL;
static char *browse_fname = NULL;

/*
 * Process callback from Dialog cancel actions.
 */
static void
cancel_callback(Widget w,
		XtPointer client_data,
		XtPointer call_data)
{
    UNUSED(w);
    UNUSED(client_data);
    UNUSED(call_data);
#if 0
    /* DON'T reset browse_fname, so that user gets the current
       value as default next time when he cancels now */
    if (browse_fname != NULL)
    {
	XtFree(browse_fname);
	browse_fname = NULL;
    }
#endif
    XtUnmanageChild(dialog);
}

/*
 * Process callback from Dialog actions.
 */

static void
accept_callback(Widget w,
		XtPointer client_data,
		XtPointer call_data)
{
    XmFileSelectionBoxCallbackStruct *fcb;
    Boolean *must_exist;
    
    UNUSED(w);

    ASSERT(client_data != NULL, "Boolean value expected in client data");
    must_exist = (Boolean *)client_data;

    if (browse_fname != NULL) {
	XtFree(browse_fname);
	browse_fname = NULL;
    }
    fcb = (XmFileSelectionBoxCallbackStruct *)call_data;

    /* get the filename from the file selection box */
    XmStringGetLtoR(fcb->value, G_charset, &browse_fname);

    if (*must_exist) {
	FILE *tmp_fp = XFOPEN(browse_fname, "r");
	dviErrFlagT errflag = NO_ERROR;
	if (tmp_fp == NULL) {
	    popup_message(XtParent(dialog),
			  MSG_ERR, NULL, "Couldn't open %s: %s\n",
			  browse_fname, strerror(errno));
	    /* leave file selection box open */
	    return;
	}
	else if (!process_preamble(tmp_fp, &errflag)
		 || !find_postamble(tmp_fp, &errflag)
		 || !read_postamble(tmp_fp, &errflag, True)) {
	    popup_message(XtParent(dialog),
			  MSG_ERR, NULL, "Error opening %s:\n%s.",
			  browse_fname, get_dvi_error(errflag));
	    fclose(tmp_fp);
	    /* leave file selection box open */
	    return;
	}
	else { /* file is OK */
	    fclose(tmp_fp);
	}
    }

    /* popdown the file selection box */
    XtUnmanageChild(dialog);
}

void raise_file_selector(void)
{
    /* dummy */
    return;
}

char *
XsraSelFile(Widget parent,
	    const char *title,
	    const char *prompt,
	    const char *ok,
	    const char *cancel,
	    const char *init_path,
	    const char *mask,
	    Boolean must_exist,
	    Widget *ret_dialog)
{
#define ARG_CNT 4
    XmString filemask = NULL;
    XmString directory = NULL;
    Arg args[ARG_CNT];
    int i = 0;
    static Boolean my_must_exist; /* so that it's safe to pass its address in callback below */
    char *retval;
    
    UNUSED(prompt);
    UNUSED(ok);
    UNUSED(cancel);
    UNUSED(init_path);

    dialog = XmCreateFileSelectionDialog(parent, "file", NULL, 0);
    /* Set ret_dialog to the DialogShell parent, not the command widget
       returned by the Motif routines.  We need the DialogShell, otherwise
       Motif will crash if ret_dialog is used as parent for popup_message()
       windows!!!
    */
    *ret_dialog = XtParent(dialog);
    XtVaSetValues(XtParent(dialog), XmNtitle, title, NULL);

    my_must_exist = must_exist;
    XtAddCallback(dialog, XmNokCallback, accept_callback, (XtPointer)&my_must_exist);
    XtAddCallback(dialog, XmNcancelCallback, cancel_callback, (XtPointer)0);
    /* We have no help in this window, so hide help button */
    XtUnmanageChild(XmFileSelectionBoxGetChild(dialog, (unsigned char)XmDIALOG_HELP_BUTTON));

    /* only show files matching our mask */
    filemask = XmStringCreateLtoR((char *)mask, G_charset);
    XtSetArg(args[i], XmNpattern, filemask); i++;

    /* set directory to last directory used */
    if (browse_fname != NULL) {
	char *path = xstrdup(browse_fname);
	char *ptr = strrchr(path, '/');
	if (ptr != NULL)
	    *ptr = '\0';
	directory = XmStringCreateLtoR(path, G_charset);
	XtSetArg(args[i], XmNdirectory, directory); i++;
	free(path);
    }

    ASSERT(i < ARG_CNT, "args list too short");
    XtSetValues(dialog, args, (Cardinal)i);

    free(filemask);
    free(directory);
    
    XtManageChild(dialog);

    /* Sit in a loop until the dialog box has gone away.
       NOTE: this seems to trigger bug #884290 (pointer grab on drag&drop)
       unless drag&drop is disabled altogether (done in xdvi.c).

       FIXME: Also, the page isn't redrawn when the popup is moved, but the
       magnifier may become active. Prevent this? We're adding a globals.events.flags |= EV_EXPOSE
       hack below so that it gets at least redrawn when the dialog is closed.
     */
    do {
	XtAppProcessEvent(XtWidgetToApplicationContext(dialog), XtIMAll);
    } while (XtIsManaged(dialog));

    /* TODO: If my_must_exist is set, check that DVI file is OK
       before closing the file selection dialog */
    if (browse_fname == NULL) {
	retval = NULL;
    }
    else {
	retval = xstrdup(browse_fname);
    }

    XtDestroyWidget(dialog);
    /* so that main window gets redrawn properly ... */
    globals.events.flags |= EV_EXPOSE;
    return retval;
#undef ARG_CNT
}

#else
/* silence `empty compilation unit' warnings */
static void bar(void); static void foo() { bar(); } static void bar(void) { foo(); }
#endif /* defined(SELFILE) && defined(MOTIF) */
